package edu.fichte.pbi.prgm.cmd;

import edu.fichte.pbi.prgm.err.runtime.CannotChangeAConstantValueRuntimeError;
import edu.fichte.pbi.prgm.err.runtime.InvalidAllocationRuntimeError;
import edu.fichte.pbi.prgm.parse.arithmetical.ArithmeticalParser;
import edu.fichte.pbi.run.app.IConsole;
import edu.fichte.pbi.util.ParserException;
import edu.fichte.pbi.vm.Machine;
import edu.fichte.pbi.vm.var.IVariable;

/**
 * Allocates the value of an expression to a variable.
 * @author Elias Groll
 * @version 2.15
 */

public class Allocation extends HotspotCompiledCommand {

  public Allocation(String varString, String calcString) {
    this.calcString = calcString;
    this.varString = varString;
  }

  private String varString;
  private String calcString;
  private IVariable var;
  private IVariable calc;
  private ArithmeticalParser parser = new ArithmeticalParser();

  
  @Override
public void execute(Machine machine, IConsole console) {
    super.execute(machine,console);
    if (var != null && calc != null) {
      try {
   
        var.setValue(calc.getValue());
      } catch (ParserException e) {
        machine.getProgram().addError(new CannotChangeAConstantValueRuntimeError(
            varString));
      }
    }
  }

  public String getVarAsString() {
    return varString;
  }

  
  public String infoMsg() {
    return "[Info]Allocating \"" + calcString + "\" to \"" + varString + "\"";
  }

  
  @Override
public String toString() {
    return "ALLOCATION";
  }

  public IVariable getVar() {
    return var;
  }

  
  @Override
protected void init(Machine machine) {
    try {
      var = machine.parseIVariable(varString);
      calc = parser.parseArithmeticalOperation(calcString, machine);
    } catch (ParserException e) {
      machine.getProgram().addError(new InvalidAllocationRuntimeError(varString
          + " = " + calcString));
    }    
  }

}
