package edu.fichte.pbi.prgm.cmd.control;

import edu.fichte.pbi.prgm.cmd.ICommand;
import edu.fichte.pbi.run.app.IConsole;
import edu.fichte.pbi.util.ParserException;
import edu.fichte.pbi.vm.Machine;
import edu.fichte.pbi.vm.var.Bit;
import edu.fichte.pbi.vm.var.ConglomerateVariable;

/**
 * Goes to a subroutine.
 * 
 * @author Elias Groll
 * @version 2.15
 */
public class Gosub implements ICommand {

  private static int numberOfGosubs = 0;

  private final int id;
  private final String reference;
  private final Goto jmp;

  /**
   * Constructs a new Gosub (subroutine-call).
   * 
   * @param reference
   *          the label which should be used as subroutine
   * @throws ParserException
   *           thrown if a program has too many subroutine-calls
   */
  public Gosub(String reference) throws ParserException {
    this.reference = reference;
    jmp = new Goto(reference);
    numberOfGosubs++;
    if (numberOfGosubs >= 16) {
      throw new ParserException("too many subroutines");
    }
    id = numberOfGosubs - 1;
  }

  
  public void execute(Machine machine, IConsole console) {
    Bit[] gosubs = ((ConglomerateVariable) machine.parseIVariable("W6"))
        .getBits();
    gosubs[getId()].setValue(1);
    jmp.execute(machine, console);
  }

  
  public String infoMsg() {
    return "[Info]Call subroutine \"" + reference + "\"";
  }

  
  @Override
public String toString() {
    return "GOSUB";
  }

  /**
   * The id is used to return to this subroutine-call.
   * 
   * @return id of this Gosub
   */
  public int getId() {
    return id;
  }

}
