package edu.fichte.pbi.prgm.cmd.num;

import edu.fichte.pbi.prgm.cmd.HotspotCompiledCommand;
import edu.fichte.pbi.prgm.err.runtime.CannotChangeAConstantValueRuntimeError;
import edu.fichte.pbi.run.app.IConsole;
import edu.fichte.pbi.util.ParserException;
import edu.fichte.pbi.vm.Machine;
import edu.fichte.pbi.vm.var.IVariable;

/**
 * Generates a pseudo-random value based on the value of a variable and stores the generated value
 * in the variable.
 * 
 * @author Elias Groll
 * @version 2.15
 */
public class Random extends HotspotCompiledCommand {

  public Random(String varString) {
    super();
    this.varString = varString;
  }

  String varString;
  IVariable var;
  java.util.Random random;

  
  @Override
public void execute(Machine machine, IConsole console) {
    super.execute(machine, console);
    if (var != null) {
      try {
        random.setSeed(var.getValue());
        int rand = Math.abs(random.nextInt())%65535 ;
        var.setValue(rand);
      } catch (ParserException e) {
        machine.getProgram().addError(new CannotChangeAConstantValueRuntimeError(varString));
      }
    }
  }

  
  public String infoMsg() {
    return "[Info]Generate a pseudo-random number on " + varString;
  }

  
  @Override
public void init(Machine machine) {
    var = machine.parseIVariable(varString);
    random = new java.util.Random();
  }
  
  @Override
	public String toString() {
		return "RANDOM";
	}

}
