package edu.fichte.pbi.run;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Scanner;

import edu.fichte.pbi.prgm.Interpreter;
import edu.fichte.pbi.prgm.parse.Lexer;
import edu.fichte.pbi.prgm.struct.Program;
import edu.fichte.pbi.run.app.IConsole;
import edu.fichte.pbi.util.Stopwatch;
import edu.fichte.pbi.vm.IRunWatcher;
import edu.fichte.pbi.vm.Machine;
import edu.fichte.pbi.vm.Settings;
import edu.fichte.pbi.vm.storage.IEEPROMMonitor;

public class PluginRunner {
	private Scanner scanner;

	public void run(File file,
			IConsole console, IEEPROMMonitor monitor,
			IRunWatcher watcher, Settings settings,
			IRunWatcher advancedWatcher) {
		try {
			compile(new String(Files.readAllBytes(Paths.get(file
					.getAbsolutePath()))), file.getAbsolutePath(), console,
					settings, monitor, watcher, advancedWatcher);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * The application of the project.
	 * 
	 * @param args
	 */
	public void compile(String code, String filepath, IConsole console,
			Settings settings, IEEPROMMonitor monitor, IRunWatcher watcher,
			IRunWatcher advancedWatcher) {
		try {
			console.printLn("PiBasic VM [Version 2.15] \n"
					+ " by Elias Groll, Johanna Reichmann.\n");

			Program program = Lexer.createProgramFromString(code);
			Machine machine = new Machine(program, settings, monitor);
			Interpreter interpreter = new Interpreter(machine, console);
			
			watcher.setMachine(machine);
			if (advancedWatcher != null) {
				advancedWatcher.setMachine(machine);
			}
			
//			String input = null;
			int execs;
			machine.setExecutionIndex(0);
			console.printLn("[Execute, " + filepath + "]");
			Stopwatch.start();
			execs = interpreter.run();
			console.printLn("[End of file, " + Stopwatch.stop()
					+ " milliseconds, " + execs + " instructions]\n");

		} catch (Exception e) {
			e.printStackTrace();
			console.printLn("[Error, compile]Something went wrong - "
					+ "please report to https://github.com/EliasGroll/pbi/issues");
		}
	}

	@SuppressWarnings("unused")
	private String getCodeFromFile(File file) throws FileNotFoundException {
		scanner = new Scanner(file);
		String retVal = "";
		while (scanner.hasNextLine()) {
			retVal += scanner.nextLine() + "\n";
		}
		return retVal;
	}

}
