package edu.fichte.pbi.prgm.cmd.control;

import edu.fichte.pbi.prgm.cmd.HotspotCompiledCommand;
import edu.fichte.pbi.prgm.err.runtime.ExpressionUnparseableRuntimeError;
import edu.fichte.pbi.prgm.parse.relational.RelationParser;
import edu.fichte.pbi.run.app.IConsole;
import edu.fichte.pbi.util.ParserException;
import edu.fichte.pbi.vm.Machine;
import edu.fichte.pbi.vm.var.IVariable;

/**
 * Evaluates an expression and goes to a label if the expression is true.
 * 
 * @author Elias Groll
 * @version 2.15
 */
public class If extends HotspotCompiledCommand {
	String expression;
	Goto jump;
	IVariable relation;
	RelationParser rp = new RelationParser();

	public If(String expression, String label) {
		this.expression = expression;
		this.jump = new Goto(label);
	}

	@Override
	public void execute(Machine machine, IConsole console) {
		super.execute(machine, console);
		if (relation.getValue() != 0) {
			jump.execute(machine, console);
		}

	}

	public String infoMsg() {
		return "[Info]If " + expression + " then resume after \""
				+ jump.getReference() + "\"";
	}

	@Override
	public String toString() {
		return "IF";
	}

	@Override
	protected void init(Machine machine) {
		try {
			relation = rp.parseRelationalOperation(expression, machine);
		} catch (ParserException e) {
			machine.getProgram().addError(
					new ExpressionUnparseableRuntimeError(expression));
		}
	}
}
