package edu.fichte.pbi.prgm.cmd.io.sound;

import java.util.ArrayList;

import edu.fichte.pbi.prgm.cmd.io.IOCommand;
import edu.fichte.pbi.prgm.war.IWarning;
import edu.fichte.pbi.run.app.IConsole;
import edu.fichte.pbi.vm.Machine;
import edu.fichte.pbi.vm.Speaker;

/**
 * Sound class which uses vm.Speaker to give out sounds
 * similar to the BS1 sounds whilst using the same syntax.
 * @author Elias Groll, Johanna Reichmann
 * @version 2.15
 */
public class Sound extends IOCommand {

	private int[] duration;
	private int[] frequency;
	private String[] noteString;
	private String[] durationString;

  /**
   * Constructor of the Sound class used to create sounds
   * similar to the BS1
   * @param num pin on which the sound shall be played on (only virtual)
   * @param note to be played (freq = 1 / 0.000095 + ((127-note) * 0,000083))
   * @param duration of the note to be played
   */
  public Sound(String num, ArrayList<String> note, ArrayList<String> duration) {
    super(num);
    this.noteString = note.toArray(new String[note.size()]);
    this.durationString = duration.toArray(new String[duration.size()]);
  }


  
  @Override
public void execute(Machine machine, IConsole console) {
    super.execute(machine, console);
    initVars(machine);
    if (dir != null) {
      int i = 0;
      machine.parseIVariable("DIR" + num).setValue(1);
      try {
    	for (i = 0; i < noteString.length; i++) {
    		Speaker.tone(frequency[i], duration[i] * 12);
    	}
      } catch (Exception e) {
        try {
          Thread.sleep(duration[i] * 12);
        } catch (InterruptedException e1) {
          machine.getProgram().addWarning(new IWarning() {
            
            public String warningMsg() {
              return "[Warn]The audio output could have been interrupted";
            }
          });
        }
      }
    }
  }
  
  private void initVars(Machine machine) {
	  this.frequency = new int[noteString.length];
	  this.duration = new int[durationString.length];
	for (int i = 0; i < noteString.length; i++) {
		System.out.println("duration[" + i + "] = " + durationString[i]);
		this.duration[i] = (int) machine.parseIVariable(durationString[i]).getValue();
		int note = (int) machine.parseIVariable(noteString[i]).getValue();
		this.frequency[i] = (int) (1 / (0.000095 + ((127 - note) * 0.000083)));		
	}
  }

  
  public String infoMsg() {
	  String info = "[INFO] Sound on P" + num + ": ";
	  for (int i = 0; i < noteString.length; i++) {
		  info += frequency[i] + "Hz for " + duration[i] * 12 + "ms, ";
	  }
	  return info.substring(0, info.length() - 1);
  }

  
  @Override
public String toString() {
    return "SOUND";
  }

}
