package edu.fichte.pbi.prgm.cmd.time;

import edu.fichte.pbi.prgm.cmd.HotspotCompiledCommand;
import edu.fichte.pbi.run.app.IConsole;
import edu.fichte.pbi.vm.Machine;
import edu.fichte.pbi.vm.var.IVariable;

/**
 * Pauses the execution (time in [ms]).
 * 
 * @author Johanna Reichmann
 * @version 2.15
 */
public class Pause extends HotspotCompiledCommand {

  public Pause(String time2) {
    this.timeString = time2;
    this.multiplier = 1;
  }

  public Pause(String time2, int multiplier) {
    this.timeString = time2;
    this.multiplier = multiplier;
  }

  protected long time;
  private IVariable timeVar;
  private final String timeString;
  private final long multiplier;

  
  @Override
public void execute(Machine machine, IConsole console) {
    super.execute(machine, console);
    time = timeVar.getValue() * multiplier;
    try {
      Thread.sleep(time);
    } catch (InterruptedException e) {
      // a failed pause is not important for the program execution - do nothing
    }

  }

  
  public String infoMsg() {
    return "[Info]Paused the execution for " + timeString + " milliseconds";
  }

  
  @Override
public String toString() {
    return "PAUSE";
  }

  
  @Override
protected void init(Machine machine) {
    timeVar = machine.parseIVariable(timeString);
  }

}
