package edu.fichte.pbi.run.app;

/**
 * Interface for all consoles the BSE can use to
 * print data to and read data from.
 * @author Elias Groll
 * @version 2.15
 */
public interface IConsole {
  /**
   * Prints a given line to the console and adds a newline.
   * @param line to be printed to the console
   */
  public void printLn(String line);

  /**
   * Prints a given line to the console,
   * but does not add a newline.
   * @param line to be printed to the console
   */
  public void print(String line);
  
  /**
   * Reads a line from the console.
   * @return line from the console which was entered by the user
   */
  public String readLn();
  
  /**
   * Clears the console.
   */
  public void clearConsole();
  
  /**
   * Prints a line in error format and adds a newline
   * @param err to be printed to the console
   */
  public void err(String err);
 
  /**
   * Prints a line in warning format and adds a newline
   * @param warn to be printed to the console
   */
  public void warn(String warn);
  
  /**
   * Prints a line in info format and adds a newline
   * @param info to be printed to the console
   */
  public void info(String info);
  
}
